import { EditorView, PluginValue, ViewUpdate, LayerMarker } from "@codemirror/view";
import { Awareness, AwarenessListener, LoroDoc, Subscription } from "loro-crdt";
import { Extension, SelectionRange, StateField } from "@codemirror/state";
export declare const loroCursorTheme: Extension;
export type AwarenessState = {
    type: "update";
    uid: string;
    cursor: {
        anchor: Uint8Array;
        head?: Uint8Array;
    };
    user?: {
        name: string;
        colorClassName: string;
    };
} | {
    type: "delete";
    uid: string;
};
export interface UserState {
    name: string;
    colorClassName: string;
}
type CursorEffect = {
    type: "update";
    peer: string;
    cursor: {
        anchor: number;
        head?: number;
    };
    user?: UserState;
} | {
    type: "delete";
    peer: string;
} | {
    type: "checkout";
    checkout: boolean;
};
export declare const remoteAwarenessAnnotation: import("@codemirror/state").AnnotationType<undefined>;
export declare const remoteAwarenessEffect: import("@codemirror/state").StateEffectType<CursorEffect>;
export declare const remoteAwarenessStateField: StateField<{
    remoteCursors: Map<string, CursorPosition>;
    isCheckout: boolean;
}>;
export declare const createCursorLayer: () => Extension;
export declare const createSelectionLayer: () => Extension;
/**
 * Renders a blinking cursor to indicate the cursor of another user.
 */
export declare class RemoteCursorMarker implements LayerMarker {
    private left;
    private top;
    private height;
    private name;
    private colorClassName;
    constructor(left: number, top: number, height: number, name: string, colorClassName: string);
    draw(): HTMLElement;
    update(elt: HTMLElement): boolean;
    adjust(element: HTMLElement): void;
    eq(other: RemoteCursorMarker): boolean;
    static createCursor(view: EditorView, position: SelectionRange, displayName: string, colorClassName: string): RemoteCursorMarker[];
    private static calculateAbsoluteCursorPosition;
}
export interface CursorPosition {
    uid: string;
    cursor: {
        anchor: number;
        head?: number;
    };
    user?: UserState;
}
export declare class AwarenessPlugin implements PluginValue {
    view: EditorView;
    doc: LoroDoc;
    user: UserState;
    awareness: Awareness<AwarenessState>;
    sub: Subscription;
    constructor(view: EditorView, doc: LoroDoc, user: UserState, awareness: Awareness<AwarenessState>);
    update(update: ViewUpdate): void;
    destroy(): void;
}
export declare class RemoteAwarenessPlugin implements PluginValue {
    view: EditorView;
    doc: LoroDoc;
    awareness: Awareness<AwarenessState>;
    _awarenessListener?: AwarenessListener;
    constructor(view: EditorView, doc: LoroDoc, awareness: Awareness<AwarenessState>);
    destroy(): void;
}
export {};
